#!/bin/csh -f
#
# Copyright (c) 2001,2003 by Oracle Corporation. All Rights Reserved. \
# $
#
#----------------------------------------------------------------------
# OHS mod_plsql/OWA Patch Driver Script
#----------------------------------------------------------------------
# This script applies mod_plsql and OWA package patches 
# Invoke "patch.csh" for help
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# Initialize default argument values
#----------------------------------------------------------------------
set sys_schema=sys
set sys_pwd=change_on_install
set conn_str=""
set patch_midtier=""
set patch_owa=""
set logf=`pwd`/patch.log
set ohome=""
set bkpname="prepatch"
setenv log_fname $logf

#----------------------------------------------------------------------
# Show the usage if no arguments were passed
#----------------------------------------------------------------------
set return_code=0
if ($#argv == 0) then
   goto usage
endif

#----------------------------------------------------------------------
# Parse the arguments
#----------------------------------------------------------------------
@ i=1
while ( $i <= $#argv )
   switch ($argv[$i])
   case -o:
      shift  
      set ohome = $argv[$i]
      shift
      breaksw
   case -m:
      shift  
      if ( $argv[$i] != "midtier" && $argv[$i] != "sysobjects" ) then
         echo ""
         echo "Script Failed"
         echo "-------------"
         echo "Error  : Invalid option '$argv[$i]' for patch mode"
         echo "Cause  : You have specified an invalid mode string"
         echo "Action : Pass a valid -m option (midtier or sysobjects)"
         echo ""
         exit 1
      endif  
      if ( $argv[$i] == "midtier") then
         set patch_midtier = $argv[$i]
      endif  
      if ( $argv[$i] == "sysobjects") then
         set patch_owa = $argv[$i]
      endif  
      shift
      breaksw
   case -p:
      shift
      set sys_pwd = $argv[$i]
      shift
      breaksw
   case -c:
      shift
      set conn_str = $argv[$i]
      shift
      breaksw
   case -l:
      shift
      set log_fname = `pwd`/$argv[$i]
      shift
      breaksw
   case -h:
      goto usage
      breaksw
   default:
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Unknown option "$argv[$i]
      echo "Cause  : You have specified an unknown option"
      echo "Action : Remove or Fix the invalid option"
      echo ""
      exit 1
    endsw
end

#----------------------------------------------------------------------
# Is the user in the right directory?
#----------------------------------------------------------------------
if !(-e "patch.csh") then
   echo ""
   echo "Script Failed"
   echo "-------------"
   echo "Error  : Invalid current directory for running the patch script"
   echo "Cause  : You have tried to execute the patch script remotely"
   echo "Action : Change your current directory to match the location of"
   echo "         the patch script"
   echo ""
   exit 1
endif 

#----------------------------------------------------------------------
# Does this location have the required files (or at least some of them)
#----------------------------------------------------------------------
if !(-e "modplsql/bin") then
   echo ""
   echo "Script Failed"
   echo "-------------"
   echo "Error  : Patch contents cannot be found"
   echo "Cause  : Patch contents do not exist in the current directory"
   echo "Action : Download the patch again, or check for missing files"
   echo ""
   exit 1
endif 

#----------------------------------------------------------------------
# Verify that user specified the mode for the install
#----------------------------------------------------------------------
if ($patch_midtier == "") then
   if ($patch_owa == "") then
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Patch script invoked without a mode"
      echo "Cause  : You did not specify the -m option"
      echo "Action : Re-run the patch script with a valid -m option"
      echo ""
      exit 1
   endif
endif 

#----------------------------------------------------------------------
# Check Oracle home argument
#----------------------------------------------------------------------
# Is ORACLE_HOME passed in?
if ($ohome == "") then
   echo ""
   echo "Script Failed"
   echo "-------------"
   echo "Error  : Oracle Home argument not specified"
   echo "Cause  : You have not specified the -o option"
   echo "Action : Pass a valid -o option for the correct ORACLE_HOME"
   echo ""
   exit 1
endif 

#----------------------------------------------------------------------
# Set platform specific values
#----------------------------------------------------------------------
set lib_dir = $ohome/lib
set modplsql_binary = $ohome/Apache/modplsql/bin/modplsql.so

#----------------------------------------------------------------------
# Is the ORACLE_HOME argument valid?
#----------------------------------------------------------------------
if !(-e $lib_dir) then
   echo ""
   echo "Script Failed"
   echo "-------------"
   echo "Error  : Oracle Home '$ohome' is invalid. $lib_dir not found"
   echo "Cause  : You have specified a non-existent Oracle Home argument"
   echo "Action : Pass a valid -o option for the correct ORACLE_HOME"
   echo ""
   exit 1
endif

#----------------------------------------------------------------------
# Validate parameters needed for patching OWA packages
#----------------------------------------------------------------------
if ($patch_owa == "sysobjects") then
   #-------------------------------------------------------------------
   # Did the user pass in a connect string
   #-------------------------------------------------------------------
    if ($conn_str  == "") then
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Database connect string not specified"
      echo "Cause  : You have not specified the -c option"
      echo "Action : Pass a valid -c option for the database connect string"
      echo ""
      exit 1
    endif
endif

#----------------------------------------------------------------------
# Setup environment for running SQL*Plus, if needed
#----------------------------------------------------------------------
if ($patch_owa == "sysobjects") then
   #-------------------------------------------------------------------
   # Setup LD_LIBRARY_PATH
   #-------------------------------------------------------------------
   if (! $?LD_LIBRARY_PATH) then
      setenv LD_LIBRARY_PATH ${ohome}/lib
   else
      setenv LD_LIBRARY_PATH ${ohome}/lib:${LD_LIBRARY_PATH}
   endif

   #-------------------------------------------------------------------
   # Setup Path
   #-------------------------------------------------------------------
   if (! $?PATH) then
      setenv PATH ${ohome}/bin
   else
      setenv PATH ${ohome}/bin:${PATH}
   endif

   #-------------------------------------------------------------------
   # Validate connect string passed down by the end user
   #-------------------------------------------------------------------
   # Validate Connect String
   sqlplus /nolog @valcstr.sql $sys_schema $sys_pwd $conn_str  >> $log_fname
   set return_code=$status
   if ($return_code != 0) then
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : DB connection failed using $sys_schema/******@$conn_str"
      echo "Cause  : Failed to connect using SYS as sysdba to the database"
      echo "Action : Check the following details"
      echo "         - $sys_schema password is correct"
      echo "         - Database connect string information is correct"
      echo "         - Database connect string can be resolved by Net"
      echo "         - Oracle Database/TNS Listener are up"
      echo "         - SQL*Plus can successfully invoke the following command"
      echo "              SQL> conn $sys_schema/******@$conn_str as sysdba"
      echo "         - Database is configured to allow remote connections as"
      echo "           sysdba. Invoke patch.csh with the -h option for more"
      echo "           information on when the database will not allow remote"
      echo "           connections, and what can be done"
      echo ""
      exit 1
   endif
endif 

#----------------------------------------------------------------------
# Validate parameters needed for patching mod_plsql
#----------------------------------------------------------------------
if ($patch_midtier == "midtier") then
   #-------------------------------------------------------------------
   # Does the ORACLE_HOME have mod_plsql installed in it?
   #-------------------------------------------------------------------
   if !(-e $modplsql_binary) then
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Oracle Home invalid. $modplsql_binary not found"
      echo "Cause  : You have specified a non-existent Oracle Home argument"
      echo "Action : Pass a valid -o option for the correct ORACLE_HOME"
      echo ""
      exit 1
   endif

   #-------------------------------------------------------------------
   # Does the ORACLE_HOME have enough privileges to write?
   #-------------------------------------------------------------------
   if !(-w $modplsql_binary) then
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Insufficient privileges to overwrite $modplsql_binary"
      echo "Cause  : Current user does not have enough privileges"
      echo "Action : Re-run the patch script as a user with sufficient"
      echo "         privileges"
      echo ""
      exit 1
   endif

   #-------------------------------------------------------------------
   # Does a backup directory already exist
   #-------------------------------------------------------------------
   set bkpdir="$ohome/Apache/modplsql/$bkpname"
   if (-e $bkpdir) then
      echo ""
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Backup directory $bkpdir already exists"
      echo "Cause  : The patch detected a previous backup directory"
      echo "Action : Remove or archive the existing backup directory"
      echo ""
      exit 1
   endif

   #-------------------------------------------------------------------
   # Create the required directories to backup stuff, and do the backup
   #-------------------------------------------------------------------
   mkdir $bkpdir
   mkdir $bkpdir/bin
   cp -Rf $ohome/Apache/modplsql/bin/* $bkpdir/bin/
   if ($status == 0) then
      if (-e modplsql/cfg) then
         mkdir $bkpdir/cfg
         cp -Rf $ohome/Apache/modplsql/cfg/* $bkpdir/cfg/
      endif
      if (-e modplsql/conf) then
         mkdir $bkpdir/conf
         cp -Rf $ohome/Apache/modplsql/conf/* $bkpdir/conf/
      endif
   endif

   #-------------------------------------------------------------------
   # Did the backup succeed?
   #-------------------------------------------------------------------
   if ($status != 0) then
      set return_code=1
      echo "Script Failed"
      echo "-------------"
      echo "Error  : Failed to backup original files to $bkpdir"
      echo "Cause  : Possible read/write privileges"
      echo "Action : Make sure that the privileges exist for the user"
      echo ""
      \rm -rf $bkpdir
      exit 1
   endif
endif

#----------------------------------------------------------------------
# Write out log file
#----------------------------------------------------------------------
echo ""
echo "" > $log_fname 

echo "Welcome to mod_plsql/OWA Patch"
echo "=============================="
echo "Welcome to mod_plsql/OWA Patch" >> $log_fname
echo "==============================" >> $log_fname

echo "" >> $log_fname
echo "The following details have been determined :" >> $log_fname
echo "" >> $log_fname
echo "Patch Mode      : "$patch_midtier $patch_owa >> $log_fname
echo "ORACLE_HOME     : "$ohome >> $log_fname
echo "PATH            : "$PATH >> $log_fname
if ($?LD_LIBRARY_PATH) then
   echo "LD_LIBRARY_PATH : "$LD_LIBRARY_PATH >> $log_fname
endif
if ($patch_owa == "sysobjects") then
   echo "Connect String  : "$conn_str >> $log_fname
endif
echo "Log File Name   : "$log_fname >> $log_fname
echo "" >> $log_fname
echo "" >> $log_fname
echo Patch Started at `date`
echo Patch Started at `date` >> $log_fname
echo =========================================== >> $log_fname

#----------------------------------------------------------------------
# Load the new OWA packages
#----------------------------------------------------------------------
if ($patch_owa == "sysobjects") then
   echo "...Installing OWA packages using script patload.sql"
   echo "...Installing OWA packages using script patload.sql" >> $log_fname
   sqlplus /nolog @patload.sql $sys_schema $sys_pwd $conn_str  >> $log_fname
   set return_code=$status
   if ($return_code != 0) then
      echo "" >> $log_fname
      echo "Error  : You are possibly running a pre-8.1.7.4 version of the database" >> $log_fname
      echo "Action : Upgrade the database to 8.1.7.4 or higher" >> $log_fname
      echo "" >> $log_fname
      goto finish
   endif
endif 

#----------------------------------------------------------------------
# Patch the midtier
#----------------------------------------------------------------------
if ($patch_midtier == "midtier") then
   echo "Backup done for files to $bkpdir"
   echo "Backup done for files to $bkpdir" >> $log_fname
   echo "...Installing mod_plsql patch"
   echo "...Installing mod_plsql patch" >> $log_fname

   #----------------------------------------------------------------------
   # Overlay newer bits on top of the older bits
   # ----------------------------------------------------------------------

   # Patch up modplsql
   cp -Rf modplsql/* $ohome/Apache/modplsql
   set return_code=$status
   if ($return_code != 0) then
      echo "   Failed to patch $ohome/Apache/modplsql/*" >> $log_fname
      echo "   Failed to patch $ohome/Apache/modplsql/*"
      goto finish
   endif
   echo "   Successful copy to $ohome/Apache/modplsql/*"
   echo "   Successful copy to $ohome/Apache/modplsql/*" >> $log_fname
endif 

finish:

echo Patch Ended at `date`
echo Patch Ended at `date` >> $log_fname

set RESULT=" Successfully "
if ($return_code != 0) then
set RESULT=" With Error(s) "
endif

echo ""
echo "" >> $log_fname
echo "**Patch Completed $RESULT **. " >> $log_fname
echo "**Patch Completed $RESULT **. " 
echo "Scan log $log_fname for any errors." 
echo ""

exit $return_code

usage:
echo ""
echo "Usage: patch.csh [-h] -m <patch_mode> [-l <log_file_name>]"
echo "                 [-p <sys_password>] [-c <connect_string>]"
echo "where"
echo ""
echo "   -m <patch_mode> specifies the patch_mode and can take values of"
echo "        midtier    : patches mod_plsql in the middle-tier"
echo "        sysobjects : patches the OWA packages in the database"
echo ""
echo "   -l <log_file_name> specifies the log filename. Default=patch.log"
echo ""
echo "   -h brings up this help screen"
echo ""
echo "For sysobjects mode, the following arguments are additionally needed"
echo ""
echo "   -p <sys_password> is the SYS schema password.Default=change_on_install"
echo ""
echo "   -c <connect_string> is the Database connect string"
echo ""
echo "Please view README.TXT for instructions for this particular patch"
echo ""
echo "How to Invoke:" 
echo "1. You can patch mod_plsql and OWA by running"
echo "   > patch.csh -m midtier -m sysobjects -p syspassword -c connect_string"
echo ""
echo "2. You can just patch mod_plsql by running" 
echo "   > patch.csh -m midtier "
echo ""
echo "3. You can just patch OWA packages by running" 
echo "   > patch.csh -m sysobjects -p syspassword -c connect_string"
echo ""
echo " Optionally, you can use -l to specify a log file other than patch.log"
echo ""
echo "Notes :"
echo "- For midtier mode, Oracle HTTP Server must be shutdown before installing"
echo "  the patch"
echo " "
echo "- For sysobjects mode, the following Oracle database parameter settings"
echo "  could disallow remote database connections as sysdba"
echo "     * O7_DICTIONARY_ACCESSIBILITY = FALSE"
echo "     * REMOTE_LOGIN_ PASSWORDFILE = NONE"
echo "  If this is the case, either allow remote connections temporarily, or"
echo "  upload the patch to the database machine and run the patch script in"
echo "  sysobjects mode on the database machine"
echo " "
echo "  For more details on these Oracle parameters, refer to the Oracle"
echo "  documentation."
echo " "
echo "- This script will backup older data to <OH>/Apache/modplsql/$bkpname "
echo "  A directory with the same name should not pre-exist"
echo " "
exit 1

